/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Copyright (C) 2012 Chris Mrwald (chris at pl2 dot com)
 * All Rights Reserved.
 * Contributed to and published at the Hunspell project
 * http://hunspell.sourceforge.net.
 *
 * Contributor(s):
 * 
 *  
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** **/

/* History
 * 2012-07-01 Chris Mrwald - created
 * 2012-07-24 Chris Mrwald - added Win utf8->utf16 and utf16->utf8 methods
 *
 */

//
// Windows only, should work with other compilers than MSVC also,
// if you can provide the according libraries in your program.
// MultiByteToWideChar() and WideCharToMultiByte() require kernelXX.dll
//

#include <windows.h>
#include <wchar.h>

#include "stringW32.h"

/*-----------------------------------------------------------------------------
s t r i n g   t o   u n i c o d e   f r o m   c o d e p a g e
-----------------------------------------------------------------------------*/

wchar_t* string_to_unicode_cp(char* psz, unsigned int codePage)
{
  static wchar_t wstr_out[MAX_STRING_LEN];

  memset(wstr_out, 0, sizeof(wstr_out));
  MultiByteToWideChar (codePage, 0, psz, -1, wstr_out, _countof(wstr_out) - 1);
  return wstr_out;
}

/*-----------------------------------------------------------------------------
s t r i n g   f r o m   u n i c o d e   t o   c o d e p a g e
-----------------------------------------------------------------------------*/

char* string_from_unicode_cp(wchar_t* psz, unsigned int codePage)
{
  static char str_out[MAX_STRING_LEN];

  memset(str_out, 0, sizeof(str_out));
  WideCharToMultiByte(codePage, 0, psz, -1, str_out, _countof(str_out) - 1, 0, 0);
  return str_out;
}

/*-----------------------------------------------------------------------------
s t r i n g   t o   u n i c o d e   utf8 -> utf16
-----------------------------------------------------------------------------*/

wchar_t* string_to_unicode(char* psz)
{
  static wchar_t wstr_out[MAX_STRING_LEN];

  memset(wstr_out, 0, sizeof(wstr_out));
  MultiByteToWideChar (CP_UTF8, 0, psz, -1, wstr_out, _countof(wstr_out) - 1);
  return wstr_out;
}

/*-----------------------------------------------------------------------------
s t r i n g   f r o m   u n i c o d e   utf16 -> utf8
-----------------------------------------------------------------------------*/

char* string_from_unicode(wchar_t* psz)
{
  static char str_out[MAX_STRING_LEN];

  memset(str_out, 0, sizeof(str_out));
  WideCharToMultiByte(CP_UTF8, 0, psz, -1, str_out, _countof(str_out) - 1, 0, 0);
  return str_out;
}

/*-----------------------------------------------------------------------------
                   s t r i n g   g e t C o d e P a g e
convert character encoding type as char string to Windows code page identifiers
code page referernces table at MSDN:
http://msdn.microsoft.com/en-us/library/windows/desktop/dd317756(v=vs.85).aspx
-----------------------------------------------------------------------------*/

unsigned int string_getCodePage(char* cpName_in)
{
  int   i = 0;
  char  cpName[16];

  if (!cpName_in || !cpName_in[0])
    return CP_ACP;  // use local codepage

  memset(cpName, 0, sizeof(cpName));
  for (i = 0; cpName_in[i] && i < _countof(cpName); i++) {
    if (cpName_in[i] == ' ' || cpName_in[i] == '/')
      cpName[i] = '-';
    else
      cpName[i] = tolower(cpName_in[i]);
  }

  if (strstr(cpName, "8859")) {
    if (strstr(cpName + 4, "-1"))
      return 28591;
    if (strstr(cpName + 4, "-2"))
      return 28592;
    if (strstr(cpName + 4, "-3"))
      return 28593;
    if (strstr(cpName + 4, "-4"))
      return 28594;
    if (strstr(cpName + 4, "-5"))
      return 28595;
    if (strstr(cpName + 4, "-6"))
      return 28596;
    if (strstr(cpName + 4, "-7"))
      return 28597;
    if (strstr(cpName + 4, "-8-i"))
      return 38598;
    if (strstr(cpName + 4, "-8"))
      return 28598;
    if (strstr(cpName + 4, "-9"))
      return 28599;
    if (strstr(cpName + 4, "-13"))
      return 28603;
    if (strstr(cpName + 4, "-15"))
      return 28605;
  }

  if (strstr(cpName, "utf")) {
    if (strstr(cpName + 3, "7"))
      return 65000;
    if (strstr(cpName + 3, "8"))
      return CP_UTF8;
    if (strstr(cpName + 3, "16"))
      return 1200;
    if (strstr(cpName + 3, "32be"))
      return 120001;
    if (strstr(cpName + 3, "32"))
      return 120000;
  }

  if (strstr(cpName, "asmo-708"))
    return 708;
  if (strstr(cpName, "cp866"))
    return 866;

  if (strstr(cpName, "windows")) {
    if (strstr(cpName + 7, "874"))
      return 874;
    if (strstr(cpName + 7, "1250"))
      return 1250;
    if (strstr(cpName + 7, "1251"))
      return 1251;
    if (strstr(cpName + 7, "1252"))
      return 1252;
    if (strstr(cpName + 7, "1253"))
      return 1253;
    if (strstr(cpName + 7, "1254"))
      return 1254;
    if (strstr(cpName + 7, "1255"))
      return 1255;
    if (strstr(cpName + 7, "1256"))
      return 1256;
    if (strstr(cpName + 7, "1257"))
      return 1257;
    if (strstr(cpName + 7, "1258"))
      return 1258;
  }

  if (strstr(cpName, "cp875"))
    return 875;
  if (strstr(cpName, "shift-jis"))
    return 932;
  if (strstr(cpName, "gb2312"))
    return 936;
  if (strstr(cpName, "ks-c-5601-1987"))
    return 949;
  if (strstr(cpName, "big5"))
    return 950;
  if (strstr(cpName, "unicodeFFFE"))
    return 1201;
  if (strstr(cpName, "johab"))
    return 1361;

  if (strstr(cpName, "chinese-cns"))
    return 20000;
  if (strstr(cpName, "cp20001"))
    return 20001;
  if (strstr(cpName, "chinese-eten"))
    return 20002;
  if (strstr(cpName, "cp20003"))
    return 20003;
  if (strstr(cpName, "cp20004"))
    return 20004;
  if (strstr(cpName, "cp20005"))
    return 20005;

  if (strstr(cpName, "koi8-u"))
    return 21866;

  if (strstr(cpName, "x-europa"))
    return 29001;
  if (strstr(cpName, "2022-jp"))
    return 50220;
  if (strstr(cpName, "2022jp"))
    return 50221;
  if (strstr(cpName, "2022-kr"))
    return 50225;
  if (strstr(cpName, "cp50227"))
    return 50227;

  if (strstr(cpName, "euc-jp"))
    return 51932;
  if (strstr(cpName, "euc-cn"))
    return 51936;
  if (strstr(cpName, "euc-kr"))
    return 51949;

  if (strstr(cpName, "hz-gb-2312"))
    return 52936;
  if (strstr(cpName, "gb18030"))
    return 54936;

  if (strstr(cpName, "iscii")) {
    if (strstr(cpName, "-de"))
      return 57002;
    if (strstr(cpName, "-be"))
      return 57003;
    if (strstr(cpName, "-ta"))
      return 57004;
    if (strstr(cpName, "-te"))
      return 57005;
    if (strstr(cpName, "-as"))
      return 57006;
    if (strstr(cpName, "-or"))
      return 57007;
    if (strstr(cpName, "-ka"))
      return 57008;
    if (strstr(cpName, "-ma"))
      return 57009;
    if (strstr(cpName, "-gu"))
      return 57010;
    if (strstr(cpName, "-pa"))
      return 57011;
  }

  // no match found, use local codepage
  return CP_ACP;
}

